# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from authentik_client.models.binding_type_enum import BindingTypeEnum
from authentik_client.models.digest_algorithm_enum import DigestAlgorithmEnum
from authentik_client.models.group_matching_mode_enum import GroupMatchingModeEnum
from authentik_client.models.name_id_policy_enum import NameIdPolicyEnum
from authentik_client.models.policy_engine_mode import PolicyEngineMode
from authentik_client.models.signature_algorithm_enum import SignatureAlgorithmEnum
from authentik_client.models.user_matching_mode_enum import UserMatchingModeEnum
from typing import Optional, Set
from typing_extensions import Self

class PatchedSAMLSourceRequest(BaseModel):
    """
    SAMLSource Serializer
    """ # noqa: E501
    name: Optional[Annotated[str, Field(min_length=1, strict=True)]] = Field(default=None, description="Source's display Name.")
    slug: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=50)]] = Field(default=None, description="Internal source name, used in URLs.")
    enabled: Optional[StrictBool] = None
    authentication_flow: Optional[StrictStr] = Field(default=None, description="Flow to use when authenticating existing users.")
    enrollment_flow: Optional[StrictStr] = Field(default=None, description="Flow to use when enrolling new users.")
    user_property_mappings: Optional[List[StrictStr]] = None
    group_property_mappings: Optional[List[StrictStr]] = None
    policy_engine_mode: Optional[PolicyEngineMode] = None
    user_matching_mode: Optional[UserMatchingModeEnum] = Field(default=None, description="How the source determines if an existing user should be authenticated or a new user enrolled.")
    user_path_template: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    group_matching_mode: Optional[GroupMatchingModeEnum] = Field(default=None, description="How the source determines if an existing group should be used or a new group created.")
    pre_authentication_flow: Optional[StrictStr] = Field(default=None, description="Flow used before authentication.")
    issuer: Optional[StrictStr] = Field(default=None, description="Also known as Entity ID. Defaults the Metadata URL.")
    sso_url: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=200)]] = Field(default=None, description="URL that the initial Login request is sent to.")
    slo_url: Optional[Annotated[str, Field(strict=True, max_length=200)]] = Field(default=None, description="Optional URL if your IDP supports Single-Logout.")
    allow_idp_initiated: Optional[StrictBool] = Field(default=None, description="Allows authentication flows initiated by the IdP. This can be a security risk, as no validation of the request ID is done.")
    name_id_policy: Optional[NameIdPolicyEnum] = Field(default=None, description="NameID Policy sent to the IdP. Can be unset, in which case no Policy is sent.")
    binding_type: Optional[BindingTypeEnum] = None
    verification_kp: Optional[StrictStr] = Field(default=None, description="When selected, incoming assertion's Signatures will be validated against this certificate. To allow unsigned Requests, leave on default.")
    signing_kp: Optional[StrictStr] = Field(default=None, description="Keypair used to sign outgoing Responses going to the Identity Provider.")
    digest_algorithm: Optional[DigestAlgorithmEnum] = None
    signature_algorithm: Optional[SignatureAlgorithmEnum] = None
    temporary_user_delete_after: Optional[Annotated[str, Field(min_length=1, strict=True)]] = Field(default=None, description="Time offset when temporary users should be deleted. This only applies if your IDP uses the NameID Format 'transient', and the user doesn't log out manually. (Format: hours=1;minutes=2;seconds=3).")
    encryption_kp: Optional[StrictStr] = Field(default=None, description="When selected, incoming assertions are encrypted by the IdP using the public key of the encryption keypair. The assertion is decrypted by the SP using the the private key.")
    __properties: ClassVar[List[str]] = ["name", "slug", "enabled", "authentication_flow", "enrollment_flow", "user_property_mappings", "group_property_mappings", "policy_engine_mode", "user_matching_mode", "user_path_template", "group_matching_mode", "pre_authentication_flow", "issuer", "sso_url", "slo_url", "allow_idp_initiated", "name_id_policy", "binding_type", "verification_kp", "signing_kp", "digest_algorithm", "signature_algorithm", "temporary_user_delete_after", "encryption_kp"]

    @field_validator('slug')
    def slug_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[-a-zA-Z0-9_]+$", value):
            raise ValueError(r"must validate the regular expression /^[-a-zA-Z0-9_]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PatchedSAMLSourceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if authentication_flow (nullable) is None
        # and model_fields_set contains the field
        if self.authentication_flow is None and "authentication_flow" in self.model_fields_set:
            _dict['authentication_flow'] = None

        # set to None if enrollment_flow (nullable) is None
        # and model_fields_set contains the field
        if self.enrollment_flow is None and "enrollment_flow" in self.model_fields_set:
            _dict['enrollment_flow'] = None

        # set to None if slo_url (nullable) is None
        # and model_fields_set contains the field
        if self.slo_url is None and "slo_url" in self.model_fields_set:
            _dict['slo_url'] = None

        # set to None if verification_kp (nullable) is None
        # and model_fields_set contains the field
        if self.verification_kp is None and "verification_kp" in self.model_fields_set:
            _dict['verification_kp'] = None

        # set to None if signing_kp (nullable) is None
        # and model_fields_set contains the field
        if self.signing_kp is None and "signing_kp" in self.model_fields_set:
            _dict['signing_kp'] = None

        # set to None if encryption_kp (nullable) is None
        # and model_fields_set contains the field
        if self.encryption_kp is None and "encryption_kp" in self.model_fields_set:
            _dict['encryption_kp'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PatchedSAMLSourceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "slug": obj.get("slug"),
            "enabled": obj.get("enabled"),
            "authentication_flow": obj.get("authentication_flow"),
            "enrollment_flow": obj.get("enrollment_flow"),
            "user_property_mappings": obj.get("user_property_mappings"),
            "group_property_mappings": obj.get("group_property_mappings"),
            "policy_engine_mode": obj.get("policy_engine_mode"),
            "user_matching_mode": obj.get("user_matching_mode"),
            "user_path_template": obj.get("user_path_template"),
            "group_matching_mode": obj.get("group_matching_mode"),
            "pre_authentication_flow": obj.get("pre_authentication_flow"),
            "issuer": obj.get("issuer"),
            "sso_url": obj.get("sso_url"),
            "slo_url": obj.get("slo_url"),
            "allow_idp_initiated": obj.get("allow_idp_initiated"),
            "name_id_policy": obj.get("name_id_policy"),
            "binding_type": obj.get("binding_type"),
            "verification_kp": obj.get("verification_kp"),
            "signing_kp": obj.get("signing_kp"),
            "digest_algorithm": obj.get("digest_algorithm"),
            "signature_algorithm": obj.get("signature_algorithm"),
            "temporary_user_delete_after": obj.get("temporary_user_delete_after"),
            "encryption_kp": obj.get("encryption_kp")
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List
from typing import Optional, Set
from typing_extensions import Self

class OpenIDConnectConfiguration(BaseModel):
    """
    rest_framework Serializer for OIDC Configuration
    """ # noqa: E501
    issuer: StrictStr
    authorization_endpoint: StrictStr
    token_endpoint: StrictStr
    userinfo_endpoint: StrictStr
    end_session_endpoint: StrictStr
    introspection_endpoint: StrictStr
    jwks_uri: StrictStr
    response_types_supported: List[StrictStr]
    id_token_signing_alg_values_supported: List[StrictStr]
    subject_types_supported: List[StrictStr]
    token_endpoint_auth_methods_supported: List[StrictStr]
    __properties: ClassVar[List[str]] = ["issuer", "authorization_endpoint", "token_endpoint", "userinfo_endpoint", "end_session_endpoint", "introspection_endpoint", "jwks_uri", "response_types_supported", "id_token_signing_alg_values_supported", "subject_types_supported", "token_endpoint_auth_methods_supported"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OpenIDConnectConfiguration from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OpenIDConnectConfiguration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "issuer": obj.get("issuer"),
            "authorization_endpoint": obj.get("authorization_endpoint"),
            "token_endpoint": obj.get("token_endpoint"),
            "userinfo_endpoint": obj.get("userinfo_endpoint"),
            "end_session_endpoint": obj.get("end_session_endpoint"),
            "introspection_endpoint": obj.get("introspection_endpoint"),
            "jwks_uri": obj.get("jwks_uri"),
            "response_types_supported": obj.get("response_types_supported"),
            "id_token_signing_alg_values_supported": obj.get("id_token_signing_alg_values_supported"),
            "subject_types_supported": obj.get("subject_types_supported"),
            "token_endpoint_auth_methods_supported": obj.get("token_endpoint_auth_methods_supported")
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict
from typing import Any, ClassVar, Dict, List, Optional
from authentik_client.models.application_request import ApplicationRequest
from authentik_client.models.model_request import ModelRequest
from authentik_client.models.provider_model_enum import ProviderModelEnum
from authentik_client.models.transaction_policy_binding_request import TransactionPolicyBindingRequest
from typing import Optional, Set
from typing_extensions import Self

class TransactionApplicationRequest(BaseModel):
    """
    Serializer for creating a provider and an application in one transaction
    """ # noqa: E501
    app: ApplicationRequest
    provider_model: ProviderModelEnum
    provider: ModelRequest
    policy_bindings: Optional[List[TransactionPolicyBindingRequest]] = None
    __properties: ClassVar[List[str]] = ["app", "provider_model", "provider", "policy_bindings"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TransactionApplicationRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of app
        if self.app:
            _dict['app'] = self.app.to_dict()
        # override the default output from pydantic by calling `to_dict()` of provider
        if self.provider:
            _dict['provider'] = self.provider.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in policy_bindings (list)
        _items = []
        if self.policy_bindings:
            for _item in self.policy_bindings:
                if _item:
                    _items.append(_item.to_dict())
            _dict['policy_bindings'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TransactionApplicationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "app": ApplicationRequest.from_dict(obj["app"]) if obj.get("app") is not None else None,
            "provider_model": obj.get("provider_model"),
            "provider": ModelRequest.from_dict(obj["provider"]) if obj.get("provider") is not None else None,
            "policy_bindings": [TransactionPolicyBindingRequest.from_dict(_item) for _item in obj["policy_bindings"]] if obj.get("policy_bindings") is not None else None
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from authentik_client.models.log_event import LogEvent
from authentik_client.models.system_task_status_enum import SystemTaskStatusEnum
from typing import Optional, Set
from typing_extensions import Self

class SystemTask(BaseModel):
    """
    Serialize TaskInfo and TaskResult
    """ # noqa: E501
    uuid: StrictStr
    name: StrictStr
    full_name: StrictStr = Field(description="Get full name with UID")
    uid: Optional[StrictStr] = None
    description: StrictStr
    start_timestamp: datetime
    finish_timestamp: datetime
    duration: Union[StrictFloat, StrictInt]
    status: SystemTaskStatusEnum
    messages: List[LogEvent]
    expires: Optional[datetime] = None
    expiring: Optional[StrictBool] = None
    __properties: ClassVar[List[str]] = ["uuid", "name", "full_name", "uid", "description", "start_timestamp", "finish_timestamp", "duration", "status", "messages", "expires", "expiring"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SystemTask from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "uuid",
            "full_name",
            "start_timestamp",
            "finish_timestamp",
            "duration",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['messages'] = _items
        # set to None if expires (nullable) is None
        # and model_fields_set contains the field
        if self.expires is None and "expires" in self.model_fields_set:
            _dict['expires'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SystemTask from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "uuid": obj.get("uuid"),
            "name": obj.get("name"),
            "full_name": obj.get("full_name"),
            "uid": obj.get("uid"),
            "description": obj.get("description"),
            "start_timestamp": obj.get("start_timestamp"),
            "finish_timestamp": obj.get("finish_timestamp"),
            "duration": obj.get("duration"),
            "status": obj.get("status"),
            "messages": [LogEvent.from_dict(_item) for _item in obj["messages"]] if obj.get("messages") is not None else None,
            "expires": obj.get("expires"),
            "expiring": obj.get("expiring")
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class LicenseFlagsEnum(str, Enum):
    """
    LicenseFlagsEnum
    """

    """
    allowed enum values
    """
    TRIAL = 'trial'
    NON_PRODUCTION = 'non_production'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LicenseFlagsEnum from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from authentik_client.models.group_member import GroupMember
from authentik_client.models.role import Role
from typing import Optional, Set
from typing_extensions import Self

class Group(BaseModel):
    """
    Group Serializer
    """ # noqa: E501
    pk: StrictStr
    num_pk: StrictInt
    name: StrictStr
    is_superuser: Optional[StrictBool] = Field(default=None, description="Users added to this group will be superusers.")
    parent: Optional[StrictStr] = None
    parent_name: Optional[StrictStr]
    users: Optional[List[StrictInt]] = None
    users_obj: Optional[List[GroupMember]]
    attributes: Optional[Dict[str, Any]] = None
    roles: Optional[List[StrictStr]] = None
    roles_obj: List[Role]
    __properties: ClassVar[List[str]] = ["pk", "num_pk", "name", "is_superuser", "parent", "parent_name", "users", "users_obj", "attributes", "roles", "roles_obj"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Group from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "pk",
            "num_pk",
            "parent_name",
            "users_obj",
            "roles_obj",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in users_obj (list)
        _items = []
        if self.users_obj:
            for _item in self.users_obj:
                if _item:
                    _items.append(_item.to_dict())
            _dict['users_obj'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in roles_obj (list)
        _items = []
        if self.roles_obj:
            for _item in self.roles_obj:
                if _item:
                    _items.append(_item.to_dict())
            _dict['roles_obj'] = _items
        # set to None if parent (nullable) is None
        # and model_fields_set contains the field
        if self.parent is None and "parent" in self.model_fields_set:
            _dict['parent'] = None

        # set to None if parent_name (nullable) is None
        # and model_fields_set contains the field
        if self.parent_name is None and "parent_name" in self.model_fields_set:
            _dict['parent_name'] = None

        # set to None if users_obj (nullable) is None
        # and model_fields_set contains the field
        if self.users_obj is None and "users_obj" in self.model_fields_set:
            _dict['users_obj'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Group from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "pk": obj.get("pk"),
            "num_pk": obj.get("num_pk"),
            "name": obj.get("name"),
            "is_superuser": obj.get("is_superuser"),
            "parent": obj.get("parent"),
            "parent_name": obj.get("parent_name"),
            "users": obj.get("users"),
            "users_obj": [GroupMember.from_dict(_item) for _item in obj["users_obj"]] if obj.get("users_obj") is not None else None,
            "attributes": obj.get("attributes"),
            "roles": obj.get("roles"),
            "roles_obj": [Role.from_dict(_item) for _item in obj["roles_obj"]] if obj.get("roles_obj") is not None else None
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from authentik_client.models.apple_challenge_response_request import AppleChallengeResponseRequest
from authentik_client.models.authenticator_duo_challenge_response_request import AuthenticatorDuoChallengeResponseRequest
from authentik_client.models.authenticator_sms_challenge_response_request import AuthenticatorSMSChallengeResponseRequest
from authentik_client.models.authenticator_static_challenge_response_request import AuthenticatorStaticChallengeResponseRequest
from authentik_client.models.authenticator_totp_challenge_response_request import AuthenticatorTOTPChallengeResponseRequest
from authentik_client.models.authenticator_validation_challenge_response_request import AuthenticatorValidationChallengeResponseRequest
from authentik_client.models.authenticator_web_authn_challenge_response_request import AuthenticatorWebAuthnChallengeResponseRequest
from authentik_client.models.auto_submit_challenge_response_request import AutoSubmitChallengeResponseRequest
from authentik_client.models.captcha_challenge_response_request import CaptchaChallengeResponseRequest
from authentik_client.models.consent_challenge_response_request import ConsentChallengeResponseRequest
from authentik_client.models.dummy_challenge_response_request import DummyChallengeResponseRequest
from authentik_client.models.email_challenge_response_request import EmailChallengeResponseRequest
from authentik_client.models.frame_challenge_response_request import FrameChallengeResponseRequest
from authentik_client.models.identification_challenge_response_request import IdentificationChallengeResponseRequest
from authentik_client.models.o_auth_device_code_challenge_response_request import OAuthDeviceCodeChallengeResponseRequest
from authentik_client.models.o_auth_device_code_finish_challenge_response_request import OAuthDeviceCodeFinishChallengeResponseRequest
from authentik_client.models.password_challenge_response_request import PasswordChallengeResponseRequest
from authentik_client.models.plex_authentication_challenge_response_request import PlexAuthenticationChallengeResponseRequest
from authentik_client.models.prompt_challenge_response_request import PromptChallengeResponseRequest
from authentik_client.models.user_login_challenge_response_request import UserLoginChallengeResponseRequest
from pydantic import StrictStr, Field
from typing import Union, List, Optional, Dict
from typing_extensions import Literal, Self

FLOWCHALLENGERESPONSEREQUEST_ONE_OF_SCHEMAS = ["AppleChallengeResponseRequest", "AuthenticatorDuoChallengeResponseRequest", "AuthenticatorSMSChallengeResponseRequest", "AuthenticatorStaticChallengeResponseRequest", "AuthenticatorTOTPChallengeResponseRequest", "AuthenticatorValidationChallengeResponseRequest", "AuthenticatorWebAuthnChallengeResponseRequest", "AutoSubmitChallengeResponseRequest", "CaptchaChallengeResponseRequest", "ConsentChallengeResponseRequest", "DummyChallengeResponseRequest", "EmailChallengeResponseRequest", "FrameChallengeResponseRequest", "IdentificationChallengeResponseRequest", "OAuthDeviceCodeChallengeResponseRequest", "OAuthDeviceCodeFinishChallengeResponseRequest", "PasswordChallengeResponseRequest", "PlexAuthenticationChallengeResponseRequest", "PromptChallengeResponseRequest", "UserLoginChallengeResponseRequest"]

class FlowChallengeResponseRequest(BaseModel):
    """
    FlowChallengeResponseRequest
    """
    # data type: AppleChallengeResponseRequest
    oneof_schema_1_validator: Optional[AppleChallengeResponseRequest] = None
    # data type: AuthenticatorDuoChallengeResponseRequest
    oneof_schema_2_validator: Optional[AuthenticatorDuoChallengeResponseRequest] = None
    # data type: AuthenticatorSMSChallengeResponseRequest
    oneof_schema_3_validator: Optional[AuthenticatorSMSChallengeResponseRequest] = None
    # data type: AuthenticatorStaticChallengeResponseRequest
    oneof_schema_4_validator: Optional[AuthenticatorStaticChallengeResponseRequest] = None
    # data type: AuthenticatorTOTPChallengeResponseRequest
    oneof_schema_5_validator: Optional[AuthenticatorTOTPChallengeResponseRequest] = None
    # data type: AuthenticatorValidationChallengeResponseRequest
    oneof_schema_6_validator: Optional[AuthenticatorValidationChallengeResponseRequest] = None
    # data type: AuthenticatorWebAuthnChallengeResponseRequest
    oneof_schema_7_validator: Optional[AuthenticatorWebAuthnChallengeResponseRequest] = None
    # data type: AutoSubmitChallengeResponseRequest
    oneof_schema_8_validator: Optional[AutoSubmitChallengeResponseRequest] = None
    # data type: CaptchaChallengeResponseRequest
    oneof_schema_9_validator: Optional[CaptchaChallengeResponseRequest] = None
    # data type: ConsentChallengeResponseRequest
    oneof_schema_10_validator: Optional[ConsentChallengeResponseRequest] = None
    # data type: DummyChallengeResponseRequest
    oneof_schema_11_validator: Optional[DummyChallengeResponseRequest] = None
    # data type: EmailChallengeResponseRequest
    oneof_schema_12_validator: Optional[EmailChallengeResponseRequest] = None
    # data type: FrameChallengeResponseRequest
    oneof_schema_13_validator: Optional[FrameChallengeResponseRequest] = None
    # data type: IdentificationChallengeResponseRequest
    oneof_schema_14_validator: Optional[IdentificationChallengeResponseRequest] = None
    # data type: OAuthDeviceCodeChallengeResponseRequest
    oneof_schema_15_validator: Optional[OAuthDeviceCodeChallengeResponseRequest] = None
    # data type: OAuthDeviceCodeFinishChallengeResponseRequest
    oneof_schema_16_validator: Optional[OAuthDeviceCodeFinishChallengeResponseRequest] = None
    # data type: PasswordChallengeResponseRequest
    oneof_schema_17_validator: Optional[PasswordChallengeResponseRequest] = None
    # data type: PlexAuthenticationChallengeResponseRequest
    oneof_schema_18_validator: Optional[PlexAuthenticationChallengeResponseRequest] = None
    # data type: PromptChallengeResponseRequest
    oneof_schema_19_validator: Optional[PromptChallengeResponseRequest] = None
    # data type: UserLoginChallengeResponseRequest
    oneof_schema_20_validator: Optional[UserLoginChallengeResponseRequest] = None
    actual_instance: Optional[Union[AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest]] = None
    one_of_schemas: List[str] = Field(default=Literal["AppleChallengeResponseRequest", "AuthenticatorDuoChallengeResponseRequest", "AuthenticatorSMSChallengeResponseRequest", "AuthenticatorStaticChallengeResponseRequest", "AuthenticatorTOTPChallengeResponseRequest", "AuthenticatorValidationChallengeResponseRequest", "AuthenticatorWebAuthnChallengeResponseRequest", "AutoSubmitChallengeResponseRequest", "CaptchaChallengeResponseRequest", "ConsentChallengeResponseRequest", "DummyChallengeResponseRequest", "EmailChallengeResponseRequest", "FrameChallengeResponseRequest", "IdentificationChallengeResponseRequest", "OAuthDeviceCodeChallengeResponseRequest", "OAuthDeviceCodeFinishChallengeResponseRequest", "PasswordChallengeResponseRequest", "PlexAuthenticationChallengeResponseRequest", "PromptChallengeResponseRequest", "UserLoginChallengeResponseRequest"])

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = FlowChallengeResponseRequest.model_construct()
        error_messages = []
        match = 0
        # validate data type: AppleChallengeResponseRequest
        if not isinstance(v, AppleChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AppleChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorDuoChallengeResponseRequest
        if not isinstance(v, AuthenticatorDuoChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorDuoChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorSMSChallengeResponseRequest
        if not isinstance(v, AuthenticatorSMSChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorSMSChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorStaticChallengeResponseRequest
        if not isinstance(v, AuthenticatorStaticChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorStaticChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorTOTPChallengeResponseRequest
        if not isinstance(v, AuthenticatorTOTPChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorTOTPChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorValidationChallengeResponseRequest
        if not isinstance(v, AuthenticatorValidationChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorValidationChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AuthenticatorWebAuthnChallengeResponseRequest
        if not isinstance(v, AuthenticatorWebAuthnChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuthenticatorWebAuthnChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: AutoSubmitChallengeResponseRequest
        if not isinstance(v, AutoSubmitChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AutoSubmitChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: CaptchaChallengeResponseRequest
        if not isinstance(v, CaptchaChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CaptchaChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: ConsentChallengeResponseRequest
        if not isinstance(v, ConsentChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ConsentChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: DummyChallengeResponseRequest
        if not isinstance(v, DummyChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DummyChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: EmailChallengeResponseRequest
        if not isinstance(v, EmailChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `EmailChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: FrameChallengeResponseRequest
        if not isinstance(v, FrameChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `FrameChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: IdentificationChallengeResponseRequest
        if not isinstance(v, IdentificationChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IdentificationChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: OAuthDeviceCodeChallengeResponseRequest
        if not isinstance(v, OAuthDeviceCodeChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `OAuthDeviceCodeChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: OAuthDeviceCodeFinishChallengeResponseRequest
        if not isinstance(v, OAuthDeviceCodeFinishChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `OAuthDeviceCodeFinishChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: PasswordChallengeResponseRequest
        if not isinstance(v, PasswordChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PasswordChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: PlexAuthenticationChallengeResponseRequest
        if not isinstance(v, PlexAuthenticationChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PlexAuthenticationChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: PromptChallengeResponseRequest
        if not isinstance(v, PromptChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PromptChallengeResponseRequest`")
        else:
            match += 1
        # validate data type: UserLoginChallengeResponseRequest
        if not isinstance(v, UserLoginChallengeResponseRequest):
            error_messages.append(f"Error! Input type `{type(v)}` is not `UserLoginChallengeResponseRequest`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in FlowChallengeResponseRequest with oneOf schemas: AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in FlowChallengeResponseRequest with oneOf schemas: AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into AppleChallengeResponseRequest
        try:
            instance.actual_instance = AppleChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorDuoChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorDuoChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorSMSChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorSMSChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorStaticChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorStaticChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorTOTPChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorTOTPChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorValidationChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorValidationChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AuthenticatorWebAuthnChallengeResponseRequest
        try:
            instance.actual_instance = AuthenticatorWebAuthnChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AutoSubmitChallengeResponseRequest
        try:
            instance.actual_instance = AutoSubmitChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CaptchaChallengeResponseRequest
        try:
            instance.actual_instance = CaptchaChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ConsentChallengeResponseRequest
        try:
            instance.actual_instance = ConsentChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DummyChallengeResponseRequest
        try:
            instance.actual_instance = DummyChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into EmailChallengeResponseRequest
        try:
            instance.actual_instance = EmailChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into FrameChallengeResponseRequest
        try:
            instance.actual_instance = FrameChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IdentificationChallengeResponseRequest
        try:
            instance.actual_instance = IdentificationChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into OAuthDeviceCodeChallengeResponseRequest
        try:
            instance.actual_instance = OAuthDeviceCodeChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into OAuthDeviceCodeFinishChallengeResponseRequest
        try:
            instance.actual_instance = OAuthDeviceCodeFinishChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PasswordChallengeResponseRequest
        try:
            instance.actual_instance = PasswordChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PlexAuthenticationChallengeResponseRequest
        try:
            instance.actual_instance = PlexAuthenticationChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PromptChallengeResponseRequest
        try:
            instance.actual_instance = PromptChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into UserLoginChallengeResponseRequest
        try:
            instance.actual_instance = UserLoginChallengeResponseRequest.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into FlowChallengeResponseRequest with oneOf schemas: AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into FlowChallengeResponseRequest with oneOf schemas: AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AppleChallengeResponseRequest, AuthenticatorDuoChallengeResponseRequest, AuthenticatorSMSChallengeResponseRequest, AuthenticatorStaticChallengeResponseRequest, AuthenticatorTOTPChallengeResponseRequest, AuthenticatorValidationChallengeResponseRequest, AuthenticatorWebAuthnChallengeResponseRequest, AutoSubmitChallengeResponseRequest, CaptchaChallengeResponseRequest, ConsentChallengeResponseRequest, DummyChallengeResponseRequest, EmailChallengeResponseRequest, FrameChallengeResponseRequest, IdentificationChallengeResponseRequest, OAuthDeviceCodeChallengeResponseRequest, OAuthDeviceCodeFinishChallengeResponseRequest, PasswordChallengeResponseRequest, PlexAuthenticationChallengeResponseRequest, PromptChallengeResponseRequest, UserLoginChallengeResponseRequest]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from authentik_client.models.authentication_enum import AuthenticationEnum
from authentik_client.models.denied_action_enum import DeniedActionEnum
from authentik_client.models.flow_designation_enum import FlowDesignationEnum
from authentik_client.models.flow_layout_enum import FlowLayoutEnum
from authentik_client.models.policy_engine_mode import PolicyEngineMode
from typing import Optional, Set
from typing_extensions import Self

class Flow(BaseModel):
    """
    Flow Serializer
    """ # noqa: E501
    pk: StrictStr
    policybindingmodel_ptr_id: StrictStr
    name: StrictStr
    slug: Annotated[str, Field(strict=True, max_length=50)] = Field(description="Visible in the URL.")
    title: StrictStr = Field(description="Shown as the Title in Flow pages.")
    designation: FlowDesignationEnum = Field(description="Decides what this Flow is used for. For example, the Authentication flow is redirect to when an un-authenticated user visits authentik.")
    background: StrictStr = Field(description="Get the URL to the background image. If the name is /static or starts with http it is returned as-is")
    stages: List[StrictStr]
    policies: List[StrictStr]
    cache_count: StrictInt = Field(description="Get count of cached flows")
    policy_engine_mode: Optional[PolicyEngineMode] = None
    compatibility_mode: Optional[StrictBool] = Field(default=None, description="Enable compatibility mode, increases compatibility with password managers on mobile devices.")
    export_url: StrictStr = Field(description="Get export URL for flow")
    layout: Optional[FlowLayoutEnum] = None
    denied_action: Optional[DeniedActionEnum] = Field(default=None, description="Configure what should happen when a flow denies access to a user.")
    authentication: Optional[AuthenticationEnum] = Field(default=None, description="Required level of authentication and authorization to access a flow.")
    __properties: ClassVar[List[str]] = ["pk", "policybindingmodel_ptr_id", "name", "slug", "title", "designation", "background", "stages", "policies", "cache_count", "policy_engine_mode", "compatibility_mode", "export_url", "layout", "denied_action", "authentication"]

    @field_validator('slug')
    def slug_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[-a-zA-Z0-9_]+$", value):
            raise ValueError(r"must validate the regular expression /^[-a-zA-Z0-9_]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Flow from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "pk",
            "policybindingmodel_ptr_id",
            "background",
            "stages",
            "policies",
            "cache_count",
            "export_url",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Flow from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "pk": obj.get("pk"),
            "policybindingmodel_ptr_id": obj.get("policybindingmodel_ptr_id"),
            "name": obj.get("name"),
            "slug": obj.get("slug"),
            "title": obj.get("title"),
            "designation": obj.get("designation"),
            "background": obj.get("background"),
            "stages": obj.get("stages"),
            "policies": obj.get("policies"),
            "cache_count": obj.get("cache_count"),
            "policy_engine_mode": obj.get("policy_engine_mode"),
            "compatibility_mode": obj.get("compatibility_mode"),
            "export_url": obj.get("export_url"),
            "layout": obj.get("layout"),
            "denied_action": obj.get("denied_action"),
            "authentication": obj.get("authentication")
        })
        return _obj





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class NotConfiguredActionEnum(str, Enum):
    """
    NotConfiguredActionEnum
    """

    """
    allowed enum values
    """
    SKIP = 'skip'
    DENY = 'deny'
    CONFIGURE = 'configure'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of NotConfiguredActionEnum from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    authentik

    Making authentication simple.

    The version of the OpenAPI document: 2024.10.5
    Contact: hello@goauthentik.io
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from authentik_client.models.group_matching_mode_enum import GroupMatchingModeEnum
from authentik_client.models.policy_engine_mode import PolicyEngineMode
from authentik_client.models.provider_type_enum import ProviderTypeEnum
from authentik_client.models.user_matching_mode_enum import UserMatchingModeEnum
from typing import Optional, Set
from typing_extensions import Self

class OAuthSourceRequest(BaseModel):
    """
    OAuth Source Serializer
    """ # noqa: E501
    name: Annotated[str, Field(min_length=1, strict=True)] = Field(description="Source's display Name.")
    slug: Annotated[str, Field(min_length=1, strict=True, max_length=50)] = Field(description="Internal source name, used in URLs.")
    enabled: Optional[StrictBool] = None
    authentication_flow: Optional[StrictStr] = Field(default=None, description="Flow to use when authenticating existing users.")
    enrollment_flow: Optional[StrictStr] = Field(default=None, description="Flow to use when enrolling new users.")
    user_property_mappings: Optional[List[StrictStr]] = None
    group_property_mappings: Optional[List[StrictStr]] = None
    policy_engine_mode: Optional[PolicyEngineMode] = None
    user_matching_mode: Optional[UserMatchingModeEnum] = Field(default=None, description="How the source determines if an existing user should be authenticated or a new user enrolled.")
    user_path_template: Optional[Annotated[str, Field(min_length=1, strict=True)]] = None
    group_matching_mode: Optional[GroupMatchingModeEnum] = Field(default=None, description="How the source determines if an existing group should be used or a new group created.")
    provider_type: ProviderTypeEnum
    request_token_url: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="URL used to request the initial token. This URL is only required for OAuth 1.")
    authorization_url: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="URL the user is redirect to to conest the flow.")
    access_token_url: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="URL used by authentik to retrieve tokens.")
    profile_url: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="URL used by authentik to get user information.")
    consumer_key: Annotated[str, Field(min_length=1, strict=True)]
    consumer_secret: Annotated[str, Field(min_length=1, strict=True)]
    additional_scopes: Optional[StrictStr] = None
    oidc_well_known_url: Optional[StrictStr] = None
    oidc_jwks_url: Optional[StrictStr] = None
    oidc_jwks: Optional[Any] = None
    __properties: ClassVar[List[str]] = ["name", "slug", "enabled", "authentication_flow", "enrollment_flow", "user_property_mappings", "group_property_mappings", "policy_engine_mode", "user_matching_mode", "user_path_template", "group_matching_mode", "provider_type", "request_token_url", "authorization_url", "access_token_url", "profile_url", "consumer_key", "consumer_secret", "additional_scopes", "oidc_well_known_url", "oidc_jwks_url", "oidc_jwks"]

    @field_validator('slug')
    def slug_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[-a-zA-Z0-9_]+$", value):
            raise ValueError(r"must validate the regular expression /^[-a-zA-Z0-9_]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OAuthSourceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if authentication_flow (nullable) is None
        # and model_fields_set contains the field
        if self.authentication_flow is None and "authentication_flow" in self.model_fields_set:
            _dict['authentication_flow'] = None

        # set to None if enrollment_flow (nullable) is None
        # and model_fields_set contains the field
        if self.enrollment_flow is None and "enrollment_flow" in self.model_fields_set:
            _dict['enrollment_flow'] = None

        # set to None if request_token_url (nullable) is None
        # and model_fields_set contains the field
        if self.request_token_url is None and "request_token_url" in self.model_fields_set:
            _dict['request_token_url'] = None

        # set to None if authorization_url (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_url is None and "authorization_url" in self.model_fields_set:
            _dict['authorization_url'] = None

        # set to None if access_token_url (nullable) is None
        # and model_fields_set contains the field
        if self.access_token_url is None and "access_token_url" in self.model_fields_set:
            _dict['access_token_url'] = None

        # set to None if profile_url (nullable) is None
        # and model_fields_set contains the field
        if self.profile_url is None and "profile_url" in self.model_fields_set:
            _dict['profile_url'] = None

        # set to None if oidc_jwks (nullable) is None
        # and model_fields_set contains the field
        if self.oidc_jwks is None and "oidc_jwks" in self.model_fields_set:
            _dict['oidc_jwks'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OAuthSourceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "slug": obj.get("slug"),
            "enabled": obj.get("enabled"),
            "authentication_flow": obj.get("authentication_flow"),
            "enrollment_flow": obj.get("enrollment_flow"),
            "user_property_mappings": obj.get("user_property_mappings"),
            "group_property_mappings": obj.get("group_property_mappings"),
            "policy_engine_mode": obj.get("policy_engine_mode"),
            "user_matching_mode": obj.get("user_matching_mode"),
            "user_path_template": obj.get("user_path_template"),
            "group_matching_mode": obj.get("group_matching_mode"),
            "provider_type": obj.get("provider_type"),
            "request_token_url": obj.get("request_token_url"),
            "authorization_url": obj.get("authorization_url"),
            "access_token_url": obj.get("access_token_url"),
            "profile_url": obj.get("profile_url"),
            "consumer_key": obj.get("consumer_key"),
            "consumer_secret": obj.get("consumer_secret"),
            "additional_scopes": obj.get("additional_scopes"),
            "oidc_well_known_url": obj.get("oidc_well_known_url"),
            "oidc_jwks_url": obj.get("oidc_jwks_url"),
            "oidc_jwks": obj.get("oidc_jwks")
        })
        return _obj


